/**
 * Copyright (c) 2011-2012 EBM WebSourcing, 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package com.ebmwebsourcing.jbi.adaptor.impl;

import static com.ebmwebsourcing.jbi.adaptor.impl.JbiConstants.JBI_NAMESPACE_URI;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

import javax.xml.namespace.QName;
import javax.xml.transform.Result;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.ow2.petals.component.api.ComponentConfiguration;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.ebmwebsourcing.easycommons.lang.UncheckedException;
import com.ebmwebsourcing.easycommons.xml.DOMHelper;
import com.ebmwebsourcing.easycommons.xml.Transformers;

/**
 * @author Marc Jambert - EBM WebSourcing
 */
public final class ComponentJbiGenerator {

	private static final Map<ComponentType, DOMSource> defaultComponentJbiUrlsByType = initializeDefaultComponentJbiUrlsByType();

	private final DOMSource jbiSource;

	private final ComponentConfiguration componentConfiguration;

	private final File installationRoot;

	public ComponentJbiGenerator(ComponentType componentType,
			ComponentConfiguration componentConfiguration, File installationRoot) {
		assert componentType != null;
		assert componentConfiguration != null;
		assert installationRoot != null;
		this.jbiSource = getDefaultComponentJbiUrl(componentType);
		this.installationRoot = installationRoot;
		this.componentConfiguration = componentConfiguration;
	}

	/*
	 * TODO : static mapping must be implemented in a separate class (revert
	 * these changes but keep map population) this cannot really be unit tested
	 * due to the special nature of this wrapping layer.
	 */

	private DOMSource getDefaultComponentJbiUrl(ComponentType componentType) {
		DOMSource domSource = defaultComponentJbiUrlsByType.get(componentType);
		if (domSource == null) {
			throw new UncheckedException(
					String.format(
							"Cannot generate jbi.xml for component of type '%s' ; no default jbi.xml found in classpath.",
							componentType));
		}
		return domSource;
	}

	private static Map<ComponentType, DOMSource> initializeDefaultComponentJbiUrlsByType() {
		Map<ComponentType, DOMSource> jbiUrlsByType = new HashMap<ComponentType, DOMSource>();
		try {
			Enumeration<URL> defaultComponentJbiUrls = ComponentJbiGenerator.class
					.getClassLoader().getResources("jbi/jbi.xml");
			while (defaultComponentJbiUrls.hasMoreElements()) {
				URL defaultComponentJbiUrl = defaultComponentJbiUrls
						.nextElement();
				DOMSource defaultJbiSource = DOMHelper
						.parseAsDOMSource(defaultComponentJbiUrl);
				Document document = (Document) defaultJbiSource.getNode();

				NodeList nodeList = document.getElementsByTagNameNS(
						"http://java.sun.com/xml/ns/jbi",
						"component-class-name");
				assert nodeList.getLength() == 1;
				String componentClassName = nodeList.item(0).getTextContent();
				jbiUrlsByType.put(
						ComponentType.fromClassName(componentClassName),
						defaultJbiSource);
			}
			return jbiUrlsByType;
		} catch (IOException ioe) {
			throw new UncheckedException(ioe);
		}
	}

	public File generate() {

		replaceJbiParametersWithComponentConfigurationParameters(jbiSource);

		// dump jbi back to installationRoot.
		File installationRootMetaInf = new File(this.installationRoot,
				"META-INF");
        installationRootMetaInf.mkdirs();
		File jbiDumpTarget = new File(installationRootMetaInf, "jbi.xml");
		dumpModifiedJbi(jbiSource, jbiDumpTarget);
		return jbiDumpTarget;

	}

	private void dumpModifiedJbi(DOMSource jbiSource, File jbiDumpTarget) {
		final Result result = new StreamResult(jbiDumpTarget);
		Transformer transformer = Transformers.takeTransformer();
		try {
			transformer.transform(jbiSource, result);
		} catch (TransformerConfigurationException tce) {
			throw new UncheckedException(tce);
		} catch (TransformerException te) {
			throw new UncheckedException(te);
		} finally {
			Transformers.releaseTransformer(transformer);
		}
	}

	private void replaceJbiParameterWithComponentConfigurationParameter(
			DOMSource jbiSource, String componentConfigurationParameterName,
			String componentConfigurationParameterValue) {
		assert componentConfigurationParameterName != null;

		final QName parameterQName = QName
				.valueOf(componentConfigurationParameterName);
		Document jbiDocument = (Document) jbiSource.getNode();
		assert jbiDocument != null;

		NodeList parameterNodeList = jbiDocument
				.getElementsByTagNameNS(parameterQName.getNamespaceURI(),
						parameterQName.getLocalPart());
		int parameterNodeListLength = parameterNodeList.getLength();
		assert parameterNodeListLength <= 1;

		boolean isTagFound = parameterNodeListLength == 1;
		if (isTagFound) {
			Node parameterNode = parameterNodeList.item(0);
			parameterNode.setTextContent(componentConfigurationParameterValue);
		} else {
			Element parameterNode = jbiDocument.createElementNS(
					parameterQName.getNamespaceURI(),
					parameterQName.getLocalPart());
			parameterNode.setTextContent(componentConfigurationParameterValue);
			NodeList componentNodeList = jbiDocument.getElementsByTagNameNS(
					JBI_NAMESPACE_URI, "");
			int componentNodeListLength = componentNodeList.getLength();
			assert componentNodeListLength == 1;

			Node componentNode = componentNodeList.item(0);
			componentNode.appendChild(parameterNode);
		}

	}

	private void replaceJbiParametersWithComponentConfigurationParameters(
			DOMSource jbiSource) {
		replaceJbiParameterWithComponentConfigurationParameter(jbiSource,
				String.format("{%s}name", JbiConstants.JBI_NAMESPACE_URI),
				componentConfiguration.getConfigurationName());
		for (String parameterName : componentConfiguration.getParameterNames()) {
			replaceJbiParameterWithComponentConfigurationParameter(jbiSource,
					parameterName,
					componentConfiguration.getParameter(parameterName));
		}
	}
}
