/**
 * Copyright (c) 2011-2012 EBM WebSourcing, 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package com.ebmwebsourcing.jbi.adaptor.impl;

import static org.junit.Assert.assertTrue;

import java.io.File;
import java.io.FileInputStream;
import java.net.URL;

import javax.xml.namespace.QName;

import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.TemporaryFolder;
import org.ow2.petals.component.api.ServiceConfiguration;
import org.ow2.petals.component.api.ServiceConfiguration.ServiceType;

import com.ebmwebsourcing.easycommons.xml.XMLComparator;

/**
 * @author Marc Jambert - EBM WebSourcing
 */
public class ServiceUnitJbiGeneratorTest {

    private static final String NAMESPACE0 = "http://testNamespace0";

    private static final String NAMESPACE1 = "http://testNamespace1";

    private static final String NAMESPACE2 = "http://testNamespace2";

    private static final QName INTERFACE0 = new QName(NAMESPACE0, "interfaceName0");

    private static final QName INTERFACE1 = new QName(NAMESPACE1, "interfaceName1");

    private static final QName INTERFACE2 = new QName(NAMESPACE2, "interfaceName2");

    private static final QName SERVICE0 = new QName(NAMESPACE0, "serviceName0");

    private static final QName SERVICE1 = new QName(NAMESPACE1, "serviceName1");

    private static final QName SERVICE2 = new QName(NAMESPACE2, "serviceName2");

    @Rule
    final public TemporaryFolder tempFolder = new TemporaryFolder();

    @Test
    public void testGenerateFromProvideServiceUnitConfigurationWithoutParameter() throws Exception {
        URL provideServiceUnitJbi0 = Thread.currentThread().getContextClassLoader()
                .getResource(
                "provide-service-unit-jbi-0.xml");

        ServiceConfiguration internalServiceConfiguration = new ServiceConfiguration(
                "testConfigurationName0", INTERFACE0, SERVICE0, "endpointName0",
                ServiceType.PROVIDE);

        testGenerateServiceUnitConfiguration(internalServiceConfiguration, provideServiceUnitJbi0);
    }

    @Test
    public void testGenerateFromProvideServiceUnitConfigurationWithParameters() throws Exception {
        URL provideServiceUnitJbi1 = Thread.currentThread().getContextClassLoader()
                .getResource(
                "provide-service-unit-jbi-1.xml");

        ServiceConfiguration internalServiceConfiguration = new ServiceConfiguration(
                "testConfigurationName1", INTERFACE1, SERVICE1, "endpointName1",
                ServiceType.PROVIDE);
        internalServiceConfiguration.setParameter("{http://parameterNamespace1}parameterName1",
                "parameterValue1");

        testGenerateServiceUnitConfiguration(internalServiceConfiguration, provideServiceUnitJbi1);
    }

    @Test
    public void testGenerateFromConsumeServiceUnitConfigurationWithoutParameter() throws Exception {
        URL provideServiceUnitJbi0 = Thread.currentThread().getContextClassLoader()
                .getResource(
                "consume-service-unit-jbi-0.xml");

        ServiceConfiguration externalServiceConfiguration = new ServiceConfiguration(
                "testConfigurationName0", INTERFACE0, SERVICE0, "endpointName0",
                ServiceType.CONSUME);

        testGenerateServiceUnitConfiguration(externalServiceConfiguration, provideServiceUnitJbi0);
    }

    @Test
    public void testGenerateFromConsumeServiceUnitConfigurationWithParameters() throws Exception {
        URL provideServiceUnitJbi1 = Thread.currentThread().getContextClassLoader()
                .getResource(
                "consume-service-unit-jbi-1.xml");

        ServiceConfiguration externalServiceConfiguration = new ServiceConfiguration(
                "testConfigurationName1", INTERFACE1, SERVICE1, "endpointName1",
                ServiceType.CONSUME);
        externalServiceConfiguration.setParameter("{http://parameterNamespace1}parameterName1",
                "parameterValue1");
        externalServiceConfiguration.setParameter("{" + JbiConstants.CDK_NAMESPACE_URI
                + "}operation", "operation1");
        externalServiceConfiguration.setParameter("{" + JbiConstants.CDK_NAMESPACE_URI
                + "}mep", "InOut");

        testGenerateServiceUnitConfiguration(externalServiceConfiguration, provideServiceUnitJbi1);
    }

    @Test
    public void testGenerateFromMultiServiceConfigurationWithParamaters() throws Exception {
        URL eipServiceUnitJbi1 = Thread.currentThread().getContextClassLoader()
                .getResource(
                "eip-service-unit-jbi-0.xml");
        ServiceConfiguration provideServiceConfiguration = new ServiceConfiguration(
                "testConfigurationName1", INTERFACE0, SERVICE0, "endpointName0",
                ServiceType.PROVIDE);
        provideServiceConfiguration.setParameter(
                "{http://petals.ow2.org/components/eip/version-2}eip", "splitter");

        ServiceConfiguration consumeServiceConfiguration1 = new ServiceConfiguration(
                "testConfigurationName2", INTERFACE1, SERVICE1, "endpointName1",
                ServiceType.CONSUME);
        consumeServiceConfiguration1.setParameter("{" + JbiConstants.CDK_NAMESPACE_URI
                + "}mep", "InOut");
        consumeServiceConfiguration1.setParameter("{" + JbiConstants.CDK_NAMESPACE_URI
                + "}operation", "operation1");

        ServiceConfiguration consumeServiceConfiguration2 = new ServiceConfiguration(
                "testConfigurationName3", INTERFACE2, SERVICE2, "endpointName2",
                ServiceType.CONSUME);
        consumeServiceConfiguration2.setParameter("{" + JbiConstants.CDK_NAMESPACE_URI
                + "}mep", "InOut");
        consumeServiceConfiguration2.setParameter("{" + JbiConstants.CDK_NAMESPACE_URI
                + "}operation", "operation2");

        provideServiceConfiguration.addServiceConfigurationDependency(consumeServiceConfiguration1);
        provideServiceConfiguration.addServiceConfigurationDependency(consumeServiceConfiguration2);

        testGenerateServiceUnitConfiguration(provideServiceConfiguration, eipServiceUnitJbi1, true);
    }

    private void testGenerateServiceUnitConfiguration(ServiceConfiguration serviceConfiguration,
            URL expectedJbiFileUrl, boolean multiServiceGeneration) throws Exception {
        File tempInstallationRoot = this.tempFolder.newFolder("installationRoot");

        ServiceUnitJbiGenerator provideServiceUnitJbiGenerator = new ServiceUnitJbiGenerator(
                serviceConfiguration, tempInstallationRoot);

        File generatedServiceUnitJbi = provideServiceUnitJbiGenerator
                .generate(multiServiceGeneration);
        
        assertTrue(XMLComparator.isEquivalent(expectedJbiFileUrl.openStream(), new FileInputStream(
                generatedServiceUnitJbi)));
    }

    private void testGenerateServiceUnitConfiguration(ServiceConfiguration serviceConfiguration,
            URL expectedJbiFileUrl) throws Exception {
        testGenerateServiceUnitConfiguration(serviceConfiguration, expectedJbiFileUrl, false);
    }
}
